; Utility to automatically run an "AUTORUN.BAS" BASIC program from a boot disk.
; If the BASIC ROM is disabled, is will be enabled and a warmstart will be performed first,
; Save executable as "AUTORUN.SYS".
; (c) JAC! - 2010/2018

open	= $03		;Command: OPEN
putb	= $0b		;Command: PUTB
close	= $0c		;Command: CLOSE

rw	= 12		;Mode: Read/write
rwauto	= 13		;Mode: Read/write with automatic RETURN 

iocb0	= $340		;IOCB base address
iccom	= iocb0+2 	;IOCB command
icbadr	= iocb0+4 	;IOCB buffer address
icblen	= iocb0+8	;IOCB buffen length
icax1	= iocb0+10 	;IOCB auxilary byte 1
icax2	= iocb0+11	;IOCB auxilary byte 2

ciov	= $e456		;CIO vector
portb	= $d301		;MMU
warmsv	= $e474		;Warm start vector

	org $2000

start	lda portb		;Enable BASIC?
	and #2
	beq basic_on		;Already on
	jsr screen_off		;Prevent garbage
	lda portb		;Enable BASIC ROM
	and #$fd
	sta portb
	jmp warmsv		;Perform warm start to properly initialize BASIC

basic_on
	jsr screen_off		;Do not display the autorun line

	ldx #0			;Channel 0
	mva #rwauto icax1,x	;Switch to automatic input mode

	mva #putb iccom,x	;Print autorun line
	mwa #line icbadr,x
	mwa #[.len line] icblen,x
	jsr ciov
	rts

editor .byte 'E:',$9b

;	BASIC command to disable automatic input mode and load BASIC program
	.local line
	.byte 125,$9b,$9b,'POKE 842,12:RUN "D1:AUTORUN.BAS"',$9b,$1c,$1c,$1c
	.endl

	.proc screen_off	;Set screen DMA off
	mva #0 559		;Will be switched on by the next GRAPHICS statement

	lda 20			;Wait one frame until screen is off
wait	cmp 20
	beq wait
	rts
	.endp

	run start
